#ifndef __GRABCB__
#define __GRABCB__

#include <dshow.h>
#include <qedit.h>

#define RECORD_TIME_MAX		30
#define RECORD_RING_MAX		90

extern void TRACE(...);

#define IMAGE_SIZE_DEF	230400
#define IMAGE_SIZE_MAX	921600

class CMySampleGrabberCB  : public ISampleGrabberCB
{
public:
	DWORD record_time[RECORD_TIME_MAX];
	int record_index;
	int record_ring_index;
	int record_readok_index;
	int record_count;
	typedef struct ST_RINGBUFF {
		DWORD	timestanp;
		long	hour;
		long	min;
		long	sec;
		long	msec;
		long	lBufferSize;
		BYTE	imgdata[IMAGE_SIZE_MAX];
	} TY_RINGBUFF;
	TY_RINGBUFF	eRINBUFF[RECORD_RING_MAX];

	int m_iframeWidth;
	int m_iframeHeigh;
	int m_iframeDepth;
	int test_counter;

	void GetNowTime( long *hour, long *min, long *sec, long *msec );

	void SetImage(unsigned char *pBuffer,int nBufferSize);
	CMySampleGrabberCB(int CameraNum, int frameWidth, int frameHeight, int frameDepth);
	virtual ~CMySampleGrabberCB();

	STDMETHODIMP_(ULONG) AddRef() { return 2; }
	STDMETHODIMP_(ULONG) Release() { return 1; }
	STDMETHODIMP QueryInterface(REFIID riid, void ** ppv)
	{
		if( riid == IID_ISampleGrabberCB || riid == IID_IUnknown ) 
		{
			*ppv = (void *) static_cast<ISampleGrabberCB*> ( this );
			return NOERROR;
		}    
		return E_NOINTERFACE;
	}
	STDMETHODIMP SampleCB( double SampleTime, IMediaSample * pSample ){
		return S_OK;
	}

	STDMETHODIMP BufferCB( double dblSampleTime, BYTE * pBuffer, long lBufferSize )
	{
		int i;
		long hour=0,min=0,sec=0,msec=0;

		if( record_index < 0 ) record_index = 0;
		if( record_count < 0 ) record_count = 0;
		if( record_ring_index < 0 ) record_ring_index = 0;
		if( test_counter < 0 ) test_counter = 0;		
		if(lBufferSize > IMAGE_SIZE_MAX) return S_OK;

		for( i = 0 ; i < lBufferSize ; i ++ ) {
			m_Buffer[i]=pBuffer[i];
		}

#ifdef _COMM_
		for( j = 0 ; j < m_iframeHeigh ; j ++ )
		{
			i = m_iframeWidth * (m_iframeHeigh-j-1) * m_iframeDepth;
			memcpy( &eRINBUFF[record_ring_index].imgdata[m_iframeWidth*j*m_iframeDepth],
				    &pBuffer[i],
					m_iframeWidth*m_iframeDepth );
		}
#endif
		for( i = 0 ; i < lBufferSize ; i ++ ) {
			eRINBUFF[record_ring_index].imgdata[i]=pBuffer[i];
		}
		eRINBUFF[record_ring_index].lBufferSize = lBufferSize;

		record_time[record_index] = timeGetTime();
		eRINBUFF[record_ring_index].timestanp = record_time[record_index];
		record_readok_index = record_ring_index;

		int record_index_next = record_index + 1;
		if(record_index_next >= RECORD_TIME_MAX){
			record_index_next = 0;
		}
		int record_ring_index_next = record_ring_index + 1;
		if(record_ring_index_next >= RECORD_RING_MAX){
			record_ring_index_next = 0;
		}
		record_count++;
//		if(record_count > RECORD_TIME_MAX){
			//printf("record_time[record_index]      = %ld\n",(long)record_time[record_index]);
			//printf("record_time[record_index_next])= %ld\n",(long)record_time[record_index_next]);
			//printf(">>>CB frame / sec = %f\n",(RECORD_TIME_MAX * 1000.0f / ((long)record_time[record_index] - (long)record_time[record_index_next])));
//		}
		record_index = record_index_next;
		record_ring_index = record_ring_index_next;

		return S_OK;
	}
private:
	int m_iCameraNumber;
	unsigned char * m_Buffer;
};

#endif // __GRABCB__
